from alpacka.pipes.ncof_pipeline import ncof_pipeline
from alpacka.pipes.tfidf_pipeline import tfidf_pipeline

class Pipeline:
    def __init__(self):
        self.ncof = ncof_pipeline()
        self.ncof.set_class_perspective(1)
        self.tfidf = tfidf_pipeline()


def main():
    pipe = Pipeline()
    pipe.ncof.print_all_methods()
    pipe.tfidf.print_all_methods()

if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
"""
Created on Mon Nov  9 09:41:26 2020

@author: Fredrik Möller

File contatins functions that performs statistical analysis on the user sepcified input data.
"""

import numpy as np
import keras as ks
from typing import Tuple
from sklearn.feature_extraction.text import TfidfVectorizer


def calc_NCOF_from_raw_data(data: list, labels: list, class_perspective: int , nr_words: int) -> Tuple[object, dict]:
    """
    Calculates the Normalized Comparative Occurence Frequency for the input data and its asociated labels
    Data: X data, list of strings
    Labels: Y data, list integer labels for the data
    Class_perspective: integer, from which class persepctive should the NCOF score be produced?
    Outputs the NCOF score for the dataset with the specified class perspective, and a list with the word index used.

    @param data: list
    @param labels: list
    @param class_perspective: int
    @param nr_words: int
    @return: ncof, dictionary : object, dict
    """

    "create a tokenizer that transforms words from letters to text"
    t = ks.preprocessing.text.Tokenizer(num_words=nr_words)
    t.fit_on_texts(data)

    "Tranform the letter sentences to integer representations"
    data_int = t.texts_to_sequences(data)

    "Create a matrix to store all word occurences for all sentences in each available class "
    rows = len(np.unique(labels))
    "cols = len +1 due to index 0 is reserved and unused in the tokenizer "
    if t.num_words is None:
        cols = len(t.index_word) + 1
    else:
        cols = nr_words + 1

    occurrences = np.zeros([rows, cols])

    "Loop through all labels and seperate sentences in regards to their class belonging"
    "Count the number of occurences of each index for each available class. "
    "Save in the occurence matrix, seperate for each class, corresponding row to each class "
    for i, lab in enumerate(labels):
        for elm in data_int[i]:
            occurrences[lab][elm] = occurrences[lab][elm] + 1

    # "Summerize the number of words in each class and normalize the ocurrence score for each class with this score"
    # occ_norm = np.zeros([rows, cols])
    # for i, elm in enumerate(occurrences):
    #     occ_norm[i] = elm / np.sum(elm)
    #
    # "make all scores except the ones from the wanted class perspective negative"
    # occ_norm = occ_norm * -1
    # occ_norm[class_perspective] = occ_norm[class_perspective] * -1
    #
    "test"
    "calculate and normalize the occurence freq for all token in all classes except the class perspective one"
    tmp_occ = np.zeros([1,cols])
    tmp_norm = 0
    for i , elm in enumerate(occurrences):
        if i != class_perspective:
            tmp_occ += elm
            tmp_norm += sum(elm)
    norm_all_else = tmp_occ/tmp_norm
    norm_perspective = occurrences[class_perspective] / sum(occurrences[class_perspective])
    ncof = norm_perspective-norm_all_else

    "OUTPUT"
    "Clculate the NCOF score by summerizing the matrix for each integer, or by 'collapsing' the rows of the matrix"
    # ncof = np.sum(occ_norm, 0)

    "A list of each words index in the NCOF score"
    index_words = t.index_word

    "removing the first index (0) in the NCOF score and index-dict since it is reserved from the keras tokenizer implementation. https://github.com/keras-team/keras-preprocessing/blob/master/keras_preprocessing/text.py"
    "this is also done to keep the indexation cosistent through the package"
    ncof = ncof[1:]

    words = [elm for elm in index_words.values()]
    dictionary = {i: words[i] for i in range(0, cols - 1)}

    return ncof, dictionary


def calc_TFIDF_from_raw_data(data: list, labels: list, nr_words: int) -> Tuple[object, dict]:
    """
    Calculates the TF-IDF scores all data what correspondinig to the classes in Labels seperately
    @param data: list
    @param labels: list
    @param nr_words: int
    @return: TF_IDF_score, dictionary: object, dict
    """


    "Seperaates the indexes of which data samples in Data that belongs to each class"
    class_indexes = []
    for elm in np.unique(labels):
        class_indexes.append([i for i, e in enumerate(labels) if e == elm])

    " create and fit a tfidf tokenizer to the entirety of the data"
    v = TfidfVectorizer(max_features=nr_words)
    v.fit(data)

    "save the tokenizer dictionary and invert it for consistent formatting"
    dictionary = v.vocabulary_
    dictionary = {i: k for k, i in dictionary.items()}

    "used to preallocate memory for matrixes"
    dict_size = len(v.vocabulary_)
    "seperate the data o the corresponding class "
    data_sep = []
    for elm in np.unique(labels):
        data_sep.append([data[i] for i in class_indexes[elm]])

    "Calculate the TF-IDF score for each class seperately and stort the score for all classes in a matrix "
    TF_IDF_score = np.zeros((len(np.unique(labels)), dict_size))
    for i, elm in enumerate(data_sep):
        "calculation of TF-IDF"
        tmp = v.transform(elm)
        "Get indices that recived a TF-IDF score"
        ind = tmp.indices
        "Store score in the matrix"
        TF_IDF_score[i][ind] = tmp.data

    "OUTPUT: an array of floats so containing all TF-IDF scores for the words in the dictionary "
    "A dictionary containing a map between the tokenizer index and words in the corpus dictionary"
    return TF_IDF_score, dictionary


def lemmatize_data(Data, ):
    pass


if __name__ == '__main__':
    pass



from typing import List
from alpacka.functions import statistic_methods as s
from alpacka.functions import presentation_functions as pf
from configparser import ConfigParser


class tfidf_pipeline:

    def __init__(self):
        self.Dot = 10
        self.Verbose = True
        self.num_words = None
    def print_all_methods(self):
        object_methods = [method_name for method_name in dir(self) if callable(getattr(self, method_name))]
        print(object_methods)

    #### Check input data type ####
    def check_data_type(self, data):
        """self function that checks if the input data type is known to be compatible with the remaining functions"""
        if type(data).__name__ != 'list':
            raise TypeError(
                f"Input type ({type(data)}) not currently supported as an input. "
                f"\n Please format the input in on of the supported formats. "
                f"\n Supported formats: list")

    #### calc_TFIDF ####
    def calc_tfidf(self, data: list, labels: list):
        """
        Calculates the TF-IDF score for the positive and negative class.
        Returns the score and dictionary used for the calculations as lists where the indexes of the elements acts as
        'keys'
        @param data: list
        @param labels: list
        @return: Score, Dict: List[List[float]] , dict
        """
        self.check_data_type(data)

        self.score, self.dict = s.calc_TFIDF_from_raw_data(data, labels, nr_words=self.get_num_words())
        if self.Verbose:
            print(f" TF-IDF score added under 'self.score' "
                  f" use self.get_--- to access the result")

    ####
    def get_score(self):
        """Returns the Tf-idf score stored in the object"""
        return self.score

    ####
    def get_dict(self):
        """Returns the dict stored in the object"""
        return self.dict

    #### Split score ####
    def split_score(self):
        """splits the TF-idf score into two categories based on the Tf-idf value for each element. Returns the
        element indexes corresponding to the following three categories. INPUT: Inliers:      elements with tf-idf
        score between mean +- 1 sigma, outliers: elements with tf-idf score greater than mean +1 sigma, The outliers
        are returned as a list of lists where index 0 is the 1-2 sigma outliers, index 1 is the 2-3 sigma outlers,
        and index 2 is 3-> outliers The outliers for the negative class is stored in [0] The outliers for the
        positive class is stored in [1] """
        score = self.get_score()
        score = score.tolist()

        self.inliers, self.outliers = pf.sigma_splitter_TF_IDF(score)
        if self.Verbose:
            print(
                f" Inliers added under 'self.Inliers' \n Positive outliers added under 'self.Pos_outliers' \n use "
                f"self.get_--- to access the result")

    ####
    def get_inliers(self):
        """Returns the Tf-idf inliers indexes stored in the object
        The inliers for the negative class is stored in [0]
        The inliers for the positive class is stored in [1]
        OUTPUT:
            type : list of lists containing integer"""
        return self.inliers

    ####
    def get_outliers(self):
        """Returns the Tf-idf outlier indexes stored  in the object
        The outliers for the negative class is stored in [0]
        The outliers for the positive class is stored in [1]
        OUTPUT:
            type : list of lists containing integer"""
        return self.outliers

    ####

    #### unique_outliers_per_class ####
    def unique_outliers_per_class(self):
        """Seperates outliers that only occures in either the positive or negative class
        sor of only works for dataset with two classes
        OUTPUT:
            outliers_unique_neg: List[List[Any]]
            outliers_unique_pos: List[List[Any]]"""
        outliers = self.get_outliers()

        neg = outliers[0]
        pos = outliers[1]

        outliers_unique_from_positive_class = []
        outliers_unique_from_negative_class = []
        for i in range(len(pos)):
            in_pos_not_neg = set(pos[i]).difference(set(neg[i]))
            in_neg_not_pos = set(neg[i]).difference(set(pos[i]))

            outliers_unique_from_positive_class.append(list(in_pos_not_neg))
            outliers_unique_from_negative_class.append(list(in_neg_not_pos))

        self.outliers_unique_neg = outliers_unique_from_negative_class
        self.outliers_unique_pos = outliers_unique_from_positive_class
        if self.Verbose:
            print(
                f" Outliers unique for the two classes added under self.outliers_unique_neg & self.outliers_unique_pos"
                f"\n use self.get_--- to access the result")

    ####
    def get_outliers_unique_neg(self):
        """Returns the ouliers for each sigma line that only appears in the negative class"""
        return self.outliers_unique_neg

    ####
    def get_outliers_unique_pos(self):
        """Returns the ouliers for each sigma line that only appears in the positive class"""
        return self.outliers_unique_pos

    #### indexes 2 words ####
    def ind_2_txt(self, lst_of_lst):
        """Converts a list of integers to the corresponding list of words (str) stored in the objects dictionary
         INPUT:
         lst: list of integers"""
        words_all = []
        for lst in lst_of_lst:
            words = pf.ind_2_txt(lst, self.get_dict())
            words_all.append(words)
        return words_all

    #### Plot ####
    def scatter(self, classes: List[str] = None):
        """
        Plots the tf-idf score stored in the object, inliers and outliers are plotted in seperate colours for clarity
        Currently only supports two classes
        @param classes: List[str]
        """
        if classes is None:
            classes = ['negative', 'positive']
        Dot = self.get_dot()
        for score, inliers, outliers, class_name in zip(self.get_score(), self.get_inliers(), self.get_outliers(),
                                                        classes):
            pf.TFIDF_plot(score, inliers, outliers, Dot, class_name)

    #### plot outliers ####
    def plot_outliers(self, score: list, outliers: List[List[List[int]]], from_class: List[str]):
        """
        Plots the score over the TFIED outliers
        @param score: list, np array with the TFIDF score for the same class as the outliers originates
        @param outliers: List[List[int]], list of list with the integer indexes of the outliers
        @param from_class: str, string with the title of the plot
        """
        pf.TFIDF_plot_outliers(score, outliers, self.get_dot(), from_class)

    #### PLOT MERGED HISTOGRAMS####
    def plot_merged_histogram(self, data: List[List[float]], bins: int = None, x_label: str = None, y_label: str = None,
                              title: str = None, legend: List[str] = None):
        """
        Plots a merged histogram of the input data
        @param data: List[List[float]]
        @param bins: int
        @param x_label: str
        @param y_label: str
        @param title: str
        @param legend: List[str]
        """
        pf.plot_merged_histogram(data, bins, x_label, y_label, title, legend)
        if self.Verbose:
            print(f"Printing merged histograms")

    #### PLOT HISTOGRAMS####
    def plot_histogram(self, data: List[float], bins: int = None, x_label: str = None, y_label: str = None,
                       title: str = None,
                       legend: List[str] = None):
        """
        Plots a histogram of the input data
        @param data: List[float]
        @param bins: int
        @param x_label: str
        @param y_label: str
        @param title: str
        @param legend: List[str]
        """
        pf.plot_histogram(data, bins, x_label, y_label, title, legend)
        if self.Verbose:
            print(f"Printing merged histograms")

    #### remove stop words ####
    def remove_stop_words(self, lst_of_lst: List[List[str]], stop_words: List[str]) -> List[List[str]]:
        """
        Removes stopwords from the inputtexts @param lst_of_lst: List[List[str]], list of list of text words in the
        format that is recieved from the methods ind_2_txt @param stop_words: List[str], List of strings containing
        the stop words @return: words_all: List[List[str]], input list with stopwords removed
        """
        words_all = []
        for lst in lst_of_lst:
            words = [w for w in lst if w not in stop_words]
            words_all.append(words)
        return words_all

    #####  CONFIG  ###
    def set_verbose(self, bol: bool= False):
        """
        Set the self variable verbose,
        @param bol: bool, standard value False
        """
        self.Verbose = bol

    def get_num_words(self):
        return self.num_words

    def set_num_words(self, new_val: int):
        """
        Set the self variable num_words,
        @param new_val: int
        """
        self.num_words = new_val

    #### DOT ####
    def set_dot(self, new_val: int):
        """
        Set the self variable dot, used during plotting
        @param new_val: int
        """
        self.Dot = new_val

    ####
    def get_dot(self):
        return self.Dot

    def print_outliers_to_terminal(self, lst: List[List[list]], sort = None):
        """
        prints the input text outliers to the terminal window with sorted into the sigma outlier
        @param path: str
        @param lst: list
        @param sort: bool
        """
        if sort:
            for elm in lst:
                elm.sort()
        else:
            pass
        sigmas = ["1", "2", "3"]
        for outliers, sigma in zip(lst, sigmas):
            if sort:
                print(f"Printing {sigma}-sigma outliers, alphabetically sorted")
            else:
                print(f"Printing {sigma}-sigma outliers")
            print(20 * "#")
            for word in outliers:
                print(f"{word}")
            print(20 * "#")


# -*- coding: utf-8 -*-
"""
Created on Mon Nov  9 15:49:26 2020

@author: Fredrik Möller


Functions separate and plot the raw metric scores produced by the different statistical functions
"""
import numpy as np
from typing import Tuple, List
import matplotlib.pyplot as plt

plt.style.use('seaborn')


def sigma_splitter(float_arr: List[float]) -> Tuple[List[List[int]], List[List[int]], List[List[int]]]:
    """
    separates the NCOF score into the 1-3 sigma outliers for the NCOF input
    @param float_arr: List[float]
    @return: inliers , pos_outliers , neg_outliers: List[List[int]], List[List[int]], List[List[int]]
    """
    "calculates the mean and std of the input score"
    mean = np.mean(float_arr)
    std = np.std(float_arr)

    "calculate which indexes that are input inliers"
    inliers = np.where(np.logical_and(float_arr >= mean - std, float_arr <= mean + std))
    inliers = inliers[0].tolist()

    "Calculates the 1-sigma postive  outliers"
    one_pos_sigma = np.where(np.logical_and(mean + std <= float_arr, float_arr < mean + 2 * std))
    "Calculates the 2-sigma postive  outliers"
    two_pos_sigma = np.where(np.logical_and(mean + 2 * std <= float_arr, float_arr < mean + 3 * std))
    "Calculates the 3-sigma postive  outliers"
    three_pos_sigma = np.where(mean + 3 * std <= float_arr)

    "Calculates the 1-sigma negative  outliers"
    one_neg_sigma = np.where(np.logical_and(mean - 2 * std < float_arr, float_arr <= mean - std))
    "Calculates the 2-sigma negative  outliers"
    two_neg_sigma = np.where(np.logical_and(mean - 3 * std < float_arr, float_arr <= mean - 2 * std))
    "Calculates the 3-sigma negative  outliers"
    three_neg_sigma = np.where(float_arr <= mean - 3 * std)

    "stores the positive outliers in a list of lists"
    pos_outliers = [one_pos_sigma[0],
                    two_pos_sigma[0],
                    three_pos_sigma[0]]
    pos_outliers = [l.tolist() for l in pos_outliers]

    "stores the negative outliers in a list of lists"
    neg_outliers = [one_neg_sigma[0],
                    two_neg_sigma[0],
                    three_neg_sigma[0]]
    neg_outliers = [l.tolist() for l in neg_outliers]

    "OUTPUT: list of indexes"
    "inliers: list of all inliers"
    "pos_outliers: list of 3 lists that corresponds to the 1,2,3 positive sigma outlers"
    "neg_outliers: list of 3 lists that corresponds to the 1,2,3 negative sigma outlers"
    return inliers, pos_outliers, neg_outliers


def sigma_splitter_TF_IDF(score: List[List[float]]) -> Tuple[List[List[List[int]]], List[List[List[int]]]]:
    """
    Additional step for sigma splitting for the TF-IDF method
    @param score: List[List[float]] , a matrix containinig the TF-IDF score for some senences
    @return:
    """
    inliers = []
    pos_outliers = []
    for lab in score:
        inliers_tmp, pos_outliers_tmp, _ = sigma_splitter(lab)
        inliers.append(inliers_tmp)
        pos_outliers.append(pos_outliers_tmp)
    "OUTPUT: list of indexes"
    "inliers: list of all inliers"
    "pos_outliers: list of lists that corresponds to the 1,2,3 positive sigma outlers for all classes"
    "neg_outliers: list of lists that corresponds to the 1,2,3 negative sigma outlers for all classes"
    return inliers, pos_outliers


def NCOF_plot(ncof_score, inliers: List[int], pos_outliers: List[List[List[int]]], neg_outliers: List[List[List[int]]],
              dot: int, class_perspective: int):
    """
    Plots the 1-3 sigma NCOF outliers
    @param ncof_score: get from 'calc_NCOF_from_raw_data' function
    @param inliers: List[int], list of indexes, get from NCOF_sigma_spliter
    @param pos_outliers: List[List[List[int]]] ,list of lists of lists of indexes, get from NCOF_sigma_spliter
    @param neg_outliers: List[List[List[int]]] ,list of lists of lists of indexes, get from NCOF_sigma_spliter
    @param dot: int, specifies the scatter dot size of the plot
    @param class_perspective: int, used during the plotting to specify the perspective of the plot
    @return:
    """

    fig, ax = plt.subplots()
    ax.set_ylabel('NCOF score')
    ax.set_xlabel('Dictionary index')
    ax.set_title(f"NCOF score in the persepctive of class: {class_perspective}")
    y = ncof_score[inliers]
    "plot outliers"
    plt.scatter(inliers, y, s=dot, c='k')
    colours = ['r', 'b', 'g']
    for i, out in enumerate(pos_outliers):
        plt.scatter(out, ncof_score[out], s=dot, c=colours[i], alpha=0.7)
    for i, out in enumerate(neg_outliers):
        plt.scatter(out, ncof_score[out], s=dot, c=colours[i], alpha=0.7)
    ax.legend(['Inliers', '1-sigma', '2-sigma', '3-sigma'])

    "Only the outliers version"
    "plot inliers"
    fig, ax = plt.subplots()
    ax.set_ylabel('NCOF score')
    ax.set_xlabel('Dictionary index')
    ax.set_title(f"NCOF score for outliers in the persepctive of class: {class_perspective}")
    "plot outliers"
    colours = ['r', 'b', 'g']
    for i, out in enumerate(pos_outliers):
        plt.scatter(out, ncof_score[out], s=dot, c=colours[i], alpha=0.7)
    for i, out in enumerate(neg_outliers):
        plt.scatter(out, ncof_score[out], s=dot, c=colours[i], alpha=0.7)
    ax.legend(['1-sigma', '2-sigma', '3-sigma'])


def TFIDF_plot(tfidf_score, inliers: List[int], outliers: List[List[List[int]]], dot: int, class_name: List[str]):
    """Plot the TF-IDF score for the specified input.
    plots seperated by what datapoints that are considered as inliers and 1-3 sigma outliers.

    @param tfidf_score: np array containing the TF-IDF score
    @param inliers: List[int], list of indexes that are considered inliers
    @param outliers: List[List[List[int]]], list of lists of indexes that are considered 1-3 sigma outliers
    @param dot: int ,scatter dot size
    @param class_name:  List[str], list containing the names of the classes the data comes from
    @return:
    """

    colours = ['r', 'b', 'g']
    fig, ax = plt.subplots()
    ax.set_ylabel(f"TF-IDF score")
    ax.set_xlabel(f"Dictionary index")
    ax.set_title(f" TF-IDF score for the {class_name} class")

    "Plot inliers"
    y = tfidf_score[inliers]
    plt.scatter(inliers, y, s=dot, c='k')
    "Plot outliers"
    for elm, c in zip(outliers, colours):
        y = tfidf_score[elm]
        plt.scatter(elm, y, s=dot, c=c)

        ax.legend(['Inliers', '1-sigma', '2-sigma', '3-sigma'])


def TFIDF_plot_outliers(tfidf_score, outliers: List[List[List[int]]], dot: int, from_class: List[str]):
    """
    Plot the TF-IDF score for the specified input.
    plots seperated by what datapoints that are considered as inliers and 1-3 sigma outliers.

    @param tfidf_score: np array containing the TF-IDF score
    @param outliers: List[List[List[int]]] , list of lists of lists of indexes that are considered 1-3 sigma outliers
    @param dot: int ,scatter dot size
    @param from_class: List[str], list containing the names of the classes the data comes from
    """

    colours = ['r', 'b', 'g']
    plt.figure()
    fig, ax = plt.subplots()
    ax.set_ylabel(f"TF-IDF score")
    ax.set_xlabel(f"Dictionary index")
    ax.set_title(f" TF-IDF outlier score for the {from_class} class")

    for elm, c in zip(outliers, colours):
        y = tfidf_score[elm]
        plt.scatter(elm, y, s=dot, c=c)
        ax.legend(['1-sigma', '2-sigma', '3-sigma'])
    plt.show()


def ind_2_txt(ind_list: List[List[int]], dictionary: dict) -> List[List[int]]:
    """
    Transform a list of integers to their text representation corresponding to the info in the dict
    @param ind_list: List[List[int]]
    @param dictionary: dict
    @return: words : List[List[str]]
    """
    "check if input list is a list of lists, else process as a single list"
    if any(isinstance(el, list) for el in ind_list):
        words = []
        for i, lst in enumerate(ind_list):
            txt_words = []

            for elm in lst:
                txt_words.append(dictionary[elm])
            words.append(txt_words)
    else:
        words = []
        for elm in ind_list:
            words.append(dictionary[elm])

    return words


def symmetric_set_difference(a: list, b: list) -> Tuple[list, list]:
    """
    Takes the symmetric set difference of two lists and presentes the results as two lists containing 'in a but not b' & 'in b but not a'
    @param a: list
    @param b: list
    @return: a_not_in_b, b_not_in_a: list, list
    """
    a_not_in_b = list(set(a).difference(b))
    b_not_in_a = list(set(b).difference(a))
    return a_not_in_b, b_not_in_a


def plot_histogram(data: List[float], bins: int = None, x_label: str = None, y_label: str = None, title: str = None,
                   legend: List[str] = None):
    """

    @param data: List[float]
    @param bins: int
    @param x_label: str
    @param y_label: str
    @param title: str
    @param legend: List[str]
    """
    if bins is None:
        bins = 500

    fig, ax = plt.subplots(1, 1)
    counts, _, _ = ax.hist(data, bins=bins)
    max_c = max(counts)
    ax.set_ylabel(y_label)
    ax.set_xlabel(x_label)
    ax.set_title(title)
    ax.legend(legend)
    ax.set_ylim(0, max_c + 0.05 * max_c)
    plt.show()


def plot_merged_histogram(data: List[List[float]], bins: int = None, x_label: str = None, y_label: str = None,
                          title: str = None, legend: List[str] = None):
    """
    @param data: List[List[float]]
    @param bins: int
    @param x_label: str
    @param y_label: str
    @param title: str
    @param legend: List[str]
    """
    if bins is None:
        bins = 500

    fig, axes = plt.subplots(2, 1)
    counts_0, _, _ = axes[0].hist(data[0], bins=bins)
    counts_1, _, _ = axes[1].hist(data[1], bins=bins)

    max_0 = max(counts_0)
    max_1 = max(counts_1)

    axes[0].set_ylabel(y_label)
    axes[1].set_ylabel(y_label)
    axes[1].set_xlabel(x_label)
    axes[0].set_title(title)

    axes[0].legend([legend[0]])
    axes[1].legend([legend[1]])

    axes[0].set_xlim(0, 1)
    axes[1].set_xlim(0, 1)

    axes[0].set_ylim(0, max_0 + 0.05 * max_0)
    axes[1].set_ylim(0, max_1 + 0.05 * max_1)

    plt.show()


if __name__ == '__main__':
    pass



from typing import List
from alpacka.functions import statistic_methods as s
from alpacka.functions import presentation_functions as pf
from configparser import ConfigParser


class ncof_pipeline:

    def __init__(self):

        self.Dot = 10
        self.Class_perspective = None
        self.Verbose = True
        self.num_words = None
    def print_all_methods(self):
        object_methods = [method_name for method_name in dir(self) if callable(getattr(self, method_name))]
        print(object_methods)

    def set_num_words(self, new_val):
        self.num_words=new_val
    def get_num_words(self):
        return self.num_words
    #### Check input data type ####
    def check_data_type(self, data):
        """self function that checks if the input data type is known to be compatible with the remaining functions"""
        if type(data).__name__ != 'list':
            raise TypeError(
                f"Input type ({type(data)}) not supported as an input. "
                f"Please format the input in on of the supported formats. "
                f"Supported formats: List")

    #### calc_NCOF ####
    def calc_ncof(self, data: List[List[str]], labels: List[int]):
        """
        Calculates the NCOF score the the input data. Returns a list with a score and a dictionary of the data.
        The score for each index is associated with the word with the same index in the dictionary

        @param data: List[List[str]]
        @param labels: List[int]
        @return: Score, Dict: List[float], dict
        """
        self.check_data_type(data)
        self.score, self.dict = s.calc_NCOF_from_raw_data(data, labels, self.get_class_perspective(), self.num_words)
        if self.Verbose:
            print(f" NCOF score added under 'self.score'"
                  f" use self.get_--- to access the result")

    ####
    def get_score(self):
        """Returns the NCOF score for the object as a list"""
        return self.score

    ####
    def get_dict(self):
        """Returns the dictionary of the object as a dict"""
        return self.dict

    #### Split score ####
    def split_score(self):
        """splits the NCOF score into three categories based on the NCOF value for each element. Returns the element
        indexes corresponding to the following three categories. Inliers:      elements with NCOF score between mean
        +- 1 sigma, Pos_outliers: elements with NCOF score greater than mean +1 sigma, Neg_outliers: elements with
        NCOF score less than mean - 1 sigma. The outliers are returned as a list of lists where index 0 is the 1-2
        sigma outliers, index 1 is the 2-3 sigma outlers, and index 2 is 3-> outliers """
        self.inliers, self.pos_outliers, self.neg_outliers = pf.sigma_splitter(self.score)
        if self.Verbose:
            print(
                f" Inliers added under 'self.inliers' \n Positive outliers added under 'self.pos_outliers' \n "
                f"Negative outliers added under 'self.neg_outliers' \n use self.get_--- to access the result")

    ####
    def get_inliers(self):
        """Returns the NCOF inliers as a list"""
        return self.inliers

    ####
    def get_pos_outliers(self):
        """Returns the Positive NCOF outliers as a list of lists"""
        return self.pos_outliers

    ####
    def get_neg_outliers(self):
        """Returns the Negative NCOF outliers as a list of lists"""
        return self.neg_outliers

    ####

    #### indexes 2 words ####
    def ind_2_txt(self, list_of_list_of_list: list) -> List[List[List]]:
        """
        Transforms a list of integer indexes to their corresponding words in the objects dictionary, Returns a list
        of list of words
        @param list_of_list_of_list: List[List[List]] @return: words_all: List[List[List]]
        """
        words_all = []
        for list in list_of_list_of_list:
            words = pf.ind_2_txt(list, self.dict)
            words_all.append(words)
        return words_all

    #### PLOT ####
    def scatter(self):
        """Creates a scatter plot of the NCOF score and parameters saved in the object"""
        pf.NCOF_plot(self.get_score(), self.get_inliers(), self.get_pos_outliers(), self.get_neg_outliers(),
                     self.get_dot(), self.get_class_perspective())

    #### PLOT histograms ####
    def plot_histogram(self, data=list, bins: int = None, x_label: str = None, y_label: str = None, title: str = None,
                       legend: List[str] = None):
        """
        Plots a histogram over the input data with the specified parameters
        @param data: list
        @param bins: int
        @param x_label: str
        @param y_label: str
        @param title: str
        @param legend: List[str]
        """
        pf.plot_histogram(data, bins, x_label, y_label, title, legend)
        if self.Verbose:
            print(f"Printing merged histograms")

    #### remove stop words ####
    def remove_stop_words(self, list_of_list: List[List[str]], stop_words: List[str]) -> List[List[str]]:
        """
        Removes stopwords from the input
        @param list_of_list: List[List[str]]
        @param stop_words: List[str]
        @return: words_all_no_stopwords: List[List[str]]
        """
        words_all_no_stopwords = []
        for lst in list_of_list:
            # words = []
            words = [w for w in lst if w not in stop_words]
            # words.append(a)
            words_all_no_stopwords.append(words)
        return words_all_no_stopwords

    # #### SAVE LIST TO TXTFILE  ####   Exists as a function now
    # def save_2_file(self, path: str, lst: list, sort: bool = True):
    #     """
    #     Saves the input list of words to a txt file at the specified path.
    #     If no file is available at path location the function creates a new, else overwrites the existing file
    #     @param path: str
    #     @param lst: list
    #     @param sort: bool
    #     """
    #     if sort:
    #         lst.sort()
    #     else:
    #         pass
    #     if os.path.isfile(path):
    #         with open(path, 'w') as file:
    #             for w in lst:
    #                 file.write(str(w) + '\n')
    #     else:
    #         with open(path, "w+") as file:
    #             for w in lst:
    #                 file.write(str(w) + '\n')

    #####  CONFIG  ###
    #### Verbose ####
    def get_verbose(self):
        """Returns the Verbose class variable"""
        return self.Verbose

    def set_verbose(self, bol: bool = False):
        """Sets the Verbose class variable"""
        self.Verbose = bol

    #### DOT ####
    def set_dot(self, new_val: int):
        """
        Sets the Dot class variable
        @param new_val: int
        """
        self.Dot = new_val

    ####
    def get_dot(self):
        """Returns the Dot class variable"""
        return self.Dot

    #### Class_perspective ####
    def set_class_perspective(self, new_val: int = 1):
        """
        Sets the Class_perspective class variable
        @param new_val: int, defult value 1
        """
        self.Class_perspective = new_val

    ####
    def get_class_perspective(self):
        """Returns the Class_perspective class variable"""
        return self.Class_perspective

    def print_outliers_to_terminal(lst: List[List[list]], sort: bool = True):
        """
        prints the input text outliers to the terminal window with sorted into the sigma outlier
        @param path: str
        @param lst: list
        @param sort: bool
        """
        if sort:
            for elm in lst:
                elm.sort()
        else:
            pass
        sigmas = ["1", "2", "3"]
        for outliers, sigma in zip(lst, sigmas):
            if sort:
                print(f"Printing {sigma}-sigma outliers, alphabetically sorted")
            else:
                print(f"Printing {sigma}-sigma outliers")
            print(20 * "#")
            for word in outliers:
                print(f"{word}")
            print(20 * "#")


from setuptools import setup, find_packages

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

install_requirements = [
    "keras",
    "sklearn",
    "matplotlib",
    "tensorflow>=2.2",
    "numpy==1.18"
]

setup(name='alpacka',
      version='0.0.75',
      description='The alpacka Python package, used to extract and visualize metadata from text data sets',
      author='Fredrik Möller',
      author_email='fredrikmoller@recordefuture.com',
      long_description=long_description,
      long_description_content_type='text/markdown',
      url='https://github.com/BernhardMoller/alpacka',
      packages=find_packages(),
      # package_data={'alpacka': ['config.ini','Stopord.txt']},
      classifiers=[
          "Programming Language :: Python :: 3",
          "License :: OSI Approved :: MIT License",
          "Operating System :: OS Independent"],
      licence='LICENCE.txt',
      python_requires='>=3.6',
      install_requires=install_requirements
      )
